package generators;

import java.util.Random;

/**
 * 
 * @author Per Kristian Fjellby
 * 
 * This class generates a valid norwegian licence plate number
 * The list of valid prefixes, i.e. 
 * SN SP SR ST SU SV SX SY SZ TA TB TC TD TE 
 * are taken from the wikipedia page on Norwegian licece plate numbers
 *
 * @see http://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Norway
 */
public class LicencePlateNumberGenerator {

	final static String[] regionalPrefixes = {"AA", "AB", "AC", "AD", "AE", "AF", "AH", "AR", "AS", "AT", "AU", "AV", "AW", "AJ", "AK", "AL", "AN", "AP", "AX", "AY", "AZ", "BA", "BB", "FN", "BC", "BD", "BE", "BF", "BH", "BJ", "BK", "BL", "BN", "BP", "BR", "BS", "BT", "BU", "BV", "BX", "BY", "BZ", "CA", "CB", "CC", "CE", "CF", "CH", "CJ", "CK", "CL", "CN", "CP", "CR", "CS", "CT", "CU", "CV", "CX", "CY", "CZ", "DA", "DB", "DC", "DD", "DE", "DF", "DH", "DJ", "DK", "DL", "DN", "DP", "DR", "DS", "DT", "DU", "DV", "DX", "DY", "DZ", "EA", "EB", "EC", "ED", "EE", "EF", "EH", "EJ", "EK", "EN", "EP", "ER", "ES", "ET", "EU", "EV", "EX", "EY", "EZ", "FA", "FB", "FC", "FD", "FE", "FS", "FT", "FU", "FV", "FX", "FY", "FZ", "HA", "HB", "HC", "HD", "HE", "HF", "HH", "HJ", "HK", "HL", "HN", "HP", "HR", "HS", "HT", "HU", "HV", "HX", "HZ", "JA", "JB", "JC", "JD", "JE", "JF", "JH", "JJ", "JK", "JL", "JN", "JP", "JR", "JS", "JT", "JU", "JV", "JX", "JY", "JZ", "KA", "KB", "KC", "KD", "KE", "KF", "KH", "KJ", "KK", "KL", "KN", "KP", "KR", "KS", "KT", "KU", "KV", "KX", "KY", "KZ", "LA", "LB", "LC", "LD", "LE", "LH", "LJ", "LK", "LL", "LN", "LP", "LR", "LH", "LJ", "LK", "LL", "LN", "LP", "LR", "LY", "ND", "NE", "NF", "NH", "NJ", "NK", "NL", "NN", "NP", "NR", "NT", "NU", "NV", "NX", "NY", "NZ", "PA", "PB", "PC", "PD", "PE", "PF", "PH", "PJ", "PK", "PL", "LF", "PN", "PP", "PR", "PS", "PT", "PU", "PV", "PX", "PY", "PZ", "RC", "RD", "RA", "RB", "RE", "RF", "RH", "RJ", "RK", "RL", "RN", "RP", "RR", "RS", "RT", "RU", "RV", "RX", "RY", "RZ", "SA", "SB", "SC", "SD", "SE", "SF", "SH", "SJ", "SK", "SL", "SN", "SP", "SR", "ST", "SU", "SV", "SX", "SY", "SZ", "TA", "TB", "TC", "TD", "TE", "TF", "TH", "TJ", "TK", "TL", "TN", "TP", "TR", "TS", "TT", "TU", "TV", "TX", "TY", "TZ", "UA", "UB", "UC", "UD", "UE", "UF", "UH", "UJ", "UK", "UL", "UN", "UP", "UR", "US", "UT", "UU", "UV", "UX", "UY", "UZ", "VA", "VB", "VC", "FP", "VD", "VE", "VF", "VH", "VJ", "VK", "VL", "VN", "VP", "VR", "VS", "VT", "VU", "VV", "VX", "VY", "VZ", "XA", "XB", "XC", "XD", "XE", "XF", "XH", "XJ", "XK", "XL", "XN", "XP", "XR", "XS", "XT", "XU", "XV", "XX", "XY", "XZ", "YA", "YB", "YC", "YD", "YE", "YF", "YH", "YJ", "YK", "YL", "YN", "YP", "YR", "YS", "YT", "YY", "YU", "YV", "YX", "FK", "YZ", "ZA", "ZB", "ZC", "ZE", "ZH", "ZK", "ZL", "ZN", "ZD", "ZF", "ZJ", "FR", "ZP", "ZR", "ZS", "ZT", "ZU", "ZV", "ZY", "ZX", "ZZ"};
	final static String[] specialPrefixes = {"CD", "EL", "GA", "HY"};
	final static int specialProbability = 1;
	final static int regionalProbabilty = 9999;


	/**
	 * 
	 * @return
	 * 1 if regular licence plate
	 * 2 if special licence plate
	 * 
	 * Assume that 1/1000 of all vehicles are special
	 * 
	 */
	public static int prefixProbabilities(){
		int value = RandomGenerators.randomInteger(0, 9999, new Random());

		if(value < 1){
			return 2;
		}
		else return 1;
	}
	
	/**
	 * 1: 78,99 % vanlige biler = 1
	 * 2: 20 % traktorer, lastebiler, motorsykler 
	 * 3: 3 % midlertidige plater
	 * 4: 0.01 % dealer plater
	 */
	public static int lengthProbabilities(){
		double random = new Random().nextDouble();
		if(random < 0.01){
			return 4;
		}
		else if(random < 0.1) {
			return 3;
		}
		else if(random < 0.3){
			return 2;
		}
		else return 1;
	}


	/**
	 * 
	 * @return ONE valid Norwegian licence plate number
	 * 
	 * The form should be:
	 * two letters followed by 5 numbers (for regular cars)
	 * two letters followed by 4 numbers (for farm equipment, trucks, and motorcycles)
	 * two letters followed by 3 numbers (temporary plates)
	 * two letters followed by 2 numbers (dealer plates)
	 * Diplomatic plates: CD <nn><3 digit sequence number>
	 * TODO: Vintage vehicles..
	 * 
	 */
	public static String generateLicenceNumber(){

		String plateNumber = "";

		/*
		 * First, select prefix
		 */

		int prefix = prefixProbabilities();
		//probabilities = 1 means regular licence plate
		String temp = "";
		if(prefix == 1){
			temp = regionalPrefixes[ RandomGenerators.randomInteger(0, regionalPrefixes.length-1, new Random()) ];
		}
		if(prefix == 2){
			temp = specialPrefixes[RandomGenerators.randomInteger(0, specialPrefixes.length-1, new Random())];
		}

		/*
		 * Generate numbers from 10000 - 99999
		 */

//		randomInteger(10000, 99999, new Random());
		int temp2 = 0;
		int suffix = lengthProbabilities(); 
		if(suffix == 1){
			temp2 = RandomGenerators.randomInteger(10000, 99999, new Random());
		}
		if(suffix == 2){
			temp2 = RandomGenerators.randomInteger(1000, 9999, new Random());
		}
		if(suffix == 3){
			temp2 = RandomGenerators.randomInteger(100, 999, new Random());
		}
		if(suffix == 4){
			temp2 = RandomGenerators.randomInteger(10, 99, new Random());
		}
		
		return plateNumber + temp + temp2;
	}

	
	
	public static void main(String[] args) {
		//		System.out.println(regionalPrefixes[10]);
		//		System.out.println(randomInteger(0, regionalPrefixes.length, new Random()));
		//		


		for (int i = 0; i < 10000; i++) {
			System.out.println(generateLicenceNumber() + " + " + i);
		}
	}

}
