package statistics;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import sql.SqlConnection;

/**
 * 
 * Package for extracting some essential statistics from the DB
 *
 *Number of passes on record
 *Number of unregistered vehicles on record
 *Percentage of heavy vehicles
 *First entry date on record
 *Last entry date on record
 *Total amount of money collected
 *Amount of money missed (worst case)
 *
 */
public class Statistics {
	private String numberOfUnregisteredPasses;
	private String percentageOfHeavyVehicles;
	String firstDateEntry;
	String lastDateEntry;
	String totalMoney;
	String moneyMissed;
	static DecimalFormat thousandSeparator = new DecimalFormat("###,###,###,###,###,###");
	static SimpleDateFormat databaseSDF = new SimpleDateFormat("yyyyMMddHHmmss");

	private String numberOfPasses;

	public String getNumberOfPasses() {
		return numberOfPasses;
	}

	public void setNumberOfPasses(String numberOfPasses) {
		this.numberOfPasses = numberOfPasses;
	}

	public String getNumberOfUnregisteredPasses() {
		return numberOfUnregisteredPasses;
	}

	public void setNumberOfUnregisteredPasses(String numberOfUnregisteredPasses) {
		this.numberOfUnregisteredPasses = numberOfUnregisteredPasses;
	}

	public String getPercentageOfHeavyVehicles() {
		return percentageOfHeavyVehicles;
	}

	public void setPercentageOfHeavyVehicles(String percentageOfHeavyVehicles) {
		this.percentageOfHeavyVehicles = percentageOfHeavyVehicles;
	}

	public String getFirstDateEntry() {
		return firstDateEntry;
	}

	public void setFirstDateEntry(String firstDateEntry) {
		this.firstDateEntry = firstDateEntry;
	}

	public String getLastDateEntry() {
		return lastDateEntry;
	}

	public void setLastDateEntry(String lastDateEntry) {
		this.lastDateEntry = lastDateEntry;
	}

	public String getTotalMoney() {
		return totalMoney;
	}

	public void setTotalMoney(String totalMoney) {
		this.totalMoney = totalMoney;
	}

	public String getMoneyMissed() {
		return moneyMissed;
	}

	public void setMoneyMissed(String moneyMissed) {
		this.moneyMissed = moneyMissed;
	}


	public Statistics(){
		getData();
	}

	public void getData(){
		Connection con = SqlConnection.connect();
		ResultSet rs;
		//COUNT(*) FROM X are VERY slow for innoDB, as it has to search the whole table.
		//Speed up by reading the last id!
//		String query = "SELECT COUNT(id) FROM AutoPASS";
		
		String query = "SELECT id FROM `AutoPASS` ORDER BY id DESC LIMIT 1";
		
		/*
		 * Get number of vehicles
		 */
		int tempCount = 0;
		try {
			rs = SqlConnection.issueQueryWithoutClosing(query, con);
			while(rs.next()){
				tempCount = rs.getInt(1);
			}
			numberOfPasses = thousandSeparator.format(tempCount);
			rs.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}

		/*
		 * Get number of unregistered vehicles
		 */
		query = "SELECT COUNT(id) FROM AutoPASS WHERE `hasTag` = 0 AND `OCRsuccess` = 0";
		try {
			rs = SqlConnection.issueQueryWithoutClosing(query, con);
			int unregisteredTempCount = 0;
			while(rs.next()){
				unregisteredTempCount = rs.getInt(1);
			}
			numberOfUnregisteredPasses = thousandSeparator.format(unregisteredTempCount);
			rs.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}

		/*
		 * Get percentage of heavy vehicles
		 */
		query = "SELECT COUNT(id) FROM AutoPASS WHERE `heavyVehicle` = 1";
		try {
			rs = SqlConnection.issueQueryWithoutClosing(query, con);
			double d = 0;
			while(rs.next()){
				d = (rs.getDouble(1) / tempCount)*100;
			}
			DecimalFormat df = new DecimalFormat("#.##");
			percentageOfHeavyVehicles = df.format(d);
			rs.close();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		/*
		 * Get first and last data entries;
		 */
		query = "SELECT timestamp FROM `AutoPASS` ORDER BY timestamp ASC LIMIT 1";
		String tempFirstTime = null;
		try {
			rs = SqlConnection.issueQueryWithoutClosing(query, con);
			while(rs.next()){
				tempFirstTime = rs.getString(1);
			}
			rs.close();
			try {
				firstDateEntry = databaseSDF.parse(tempFirstTime).toString();
			} catch (ParseException e) {
				e.printStackTrace();
			}
		} catch (SQLException e1) {
			e1.printStackTrace();
		}
		query = "SELECT timestamp FROM `AutoPASS` ORDER BY timestamp DESC LIMIT 1";
		String tempLastTime = null;
		try {
			rs = SqlConnection.issueQueryWithoutClosing(query, con);
			while(rs.next()){
				tempLastTime = rs.getString(1);
			}
			rs.close();
			try {
				lastDateEntry = databaseSDF.parse(tempLastTime).toString();
			} catch (ParseException e) {
				e.printStackTrace();
			}
		} catch (SQLException e1) {
			e1.printStackTrace();
		}
		
		/*
		 * Get sum price
		 */
		query = "SELECT SUM(price) from AutoPASS";
		try {
			rs = SqlConnection.issueQueryWithoutClosing(query, con);
			double money = 0;
			while(rs.next()){
				money = rs.getDouble(1);
			}
			rs.close();
			totalMoney = thousandSeparator.format(money);
		} catch (SQLException e) {
			e.printStackTrace();
		}

		/*
		 * Close the connection
		 */
		try {
			con.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}

	}

	public static void main(String[] args) {
		Statistics stat = new Statistics();
		System.out.println(stat.getNumberOfPasses());
		System.out.println(stat.getPercentageOfHeavyVehicles());
		System.out.println(stat.getTotalMoney());
	}
}
